'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Define a base error class to extend.
 *
 * @type {StructError}
 */

var StructError = function (_Error) {
  _inherits(StructError, _Error);

  function StructError(message, data) {
    _classCallCheck(this, StructError);

    var _this = _possibleConstructorReturn(this, (StructError.__proto__ || Object.getPrototypeOf(StructError)).call(this, message));

    _this.name = 'StructError';

    for (var key in data) {
      _this[key] = data[key];
    }

    Error.captureStackTrace(_this, _this.constructor);
    return _this;
  }

  return StructError;
}(Error);

/**
 * Define specific errors.
 *
 * @type {StructError}
 */

var ElementInvalidError = function (_StructError) {
  _inherits(ElementInvalidError, _StructError);

  function ElementInvalidError(_ref) {
    var type = _ref.type,
        index = _ref.index,
        value = _ref.value,
        _ref$path = _ref.path,
        path = _ref$path === undefined ? [] : _ref$path;

    _classCallCheck(this, ElementInvalidError);

    var message = 'The element at `' + index + '` in an array was invalid. It should be of type "' + type + '", but it was: ' + value;
    var code = 'element_invalid';

    var _this2 = _possibleConstructorReturn(this, (ElementInvalidError.__proto__ || Object.getPrototypeOf(ElementInvalidError)).call(this, message, { code: code, type: type, path: path, index: index, value: value }));

    Error.captureStackTrace(_this2, _this2.constructor);
    return _this2;
  }

  return ElementInvalidError;
}(StructError);

var PropertyInvalidError = function (_StructError2) {
  _inherits(PropertyInvalidError, _StructError2);

  function PropertyInvalidError(_ref2) {
    var type = _ref2.type,
        key = _ref2.key,
        value = _ref2.value,
        _ref2$path = _ref2.path,
        path = _ref2$path === undefined ? [] : _ref2$path;

    _classCallCheck(this, PropertyInvalidError);

    var message = 'The `' + key + '` property in an object was invalid. It should be of type "' + type + '", but it was: ' + value;
    var code = 'property_invalid';

    var _this3 = _possibleConstructorReturn(this, (PropertyInvalidError.__proto__ || Object.getPrototypeOf(PropertyInvalidError)).call(this, message, { code: code, type: type, path: path, key: key, value: value }));

    Error.captureStackTrace(_this3, _this3.constructor);
    return _this3;
  }

  return PropertyInvalidError;
}(StructError);

var PropertyRequiredError = function (_StructError3) {
  _inherits(PropertyRequiredError, _StructError3);

  function PropertyRequiredError(_ref3) {
    var type = _ref3.type,
        key = _ref3.key,
        _ref3$path = _ref3.path,
        path = _ref3$path === undefined ? [] : _ref3$path;

    _classCallCheck(this, PropertyRequiredError);

    var message = 'The `' + key + '` property is required but was not defined. It should be of type "' + type + '".';
    var code = 'property_required';

    var _this4 = _possibleConstructorReturn(this, (PropertyRequiredError.__proto__ || Object.getPrototypeOf(PropertyRequiredError)).call(this, message, { code: code, type: type, path: path, key: key }));

    Error.captureStackTrace(_this4, _this4.constructor);
    return _this4;
  }

  return PropertyRequiredError;
}(StructError);

var PropertyUnknownError = function (_StructError4) {
  _inherits(PropertyUnknownError, _StructError4);

  function PropertyUnknownError(_ref4) {
    var key = _ref4.key,
        _ref4$path = _ref4.path,
        path = _ref4$path === undefined ? [] : _ref4$path;

    _classCallCheck(this, PropertyUnknownError);

    var message = 'The `' + key + '` property in an object was not recognized.';
    var code = 'property_unknown';

    var _this5 = _possibleConstructorReturn(this, (PropertyUnknownError.__proto__ || Object.getPrototypeOf(PropertyUnknownError)).call(this, message, { code: code, path: path, key: key }));

    Error.captureStackTrace(_this5, _this5.constructor);
    return _this5;
  }

  return PropertyUnknownError;
}(StructError);

var ValueInvalidError = function (_StructError5) {
  _inherits(ValueInvalidError, _StructError5);

  function ValueInvalidError(_ref5) {
    var type = _ref5.type,
        value = _ref5.value,
        _ref5$path = _ref5.path,
        path = _ref5$path === undefined ? [] : _ref5$path;

    _classCallCheck(this, ValueInvalidError);

    var message = 'Expected the value "' + value + '" to match type "' + type + '".';
    var code = 'value_invalid';

    var _this6 = _possibleConstructorReturn(this, (ValueInvalidError.__proto__ || Object.getPrototypeOf(ValueInvalidError)).call(this, message, { code: code, type: type, path: path, value: value }));

    Error.captureStackTrace(_this6, _this6.constructor);
    return _this6;
  }

  return ValueInvalidError;
}(StructError);

var ValueRequiredError = function (_StructError6) {
  _inherits(ValueRequiredError, _StructError6);

  function ValueRequiredError(_ref6) {
    var type = _ref6.type,
        _ref6$path = _ref6.path,
        path = _ref6$path === undefined ? [] : _ref6$path;

    _classCallCheck(this, ValueRequiredError);

    var message = 'A required property was not defined. It should be of type "' + type + '".';
    var code = 'value_required';

    var _this7 = _possibleConstructorReturn(this, (ValueRequiredError.__proto__ || Object.getPrototypeOf(ValueRequiredError)).call(this, message, { code: code, type: type, path: path }));

    Error.captureStackTrace(_this7, _this7.constructor);
    return _this7;
  }

  return ValueRequiredError;
}(StructError);

/**
 * Export.
 *
 * @type {Object}
 */

exports.ElementInvalidError = ElementInvalidError;
exports.PropertyInvalidError = PropertyInvalidError;
exports.PropertyRequiredError = PropertyRequiredError;
exports.PropertyUnknownError = PropertyUnknownError;
exports.ValueInvalidError = ValueInvalidError;
exports.ValueRequiredError = ValueRequiredError;